require("chartdirector")

class TracklabelController < ApplicationController
    include ChartDirector::InteractiveChartSupport

    def index()
        @ctrl_file = File.expand_path(__FILE__)

        # Data for the chart as 3 random data series
        r = ChartDirector::RanSeries.new(127)
        data0 = r.getSeries(100, 100, -15, 15)
        data1 = r.getSeries(100, 150, -15, 15)
        data2 = r.getSeries(100, 200, -15, 15)
        timeStamps = r.getDateSeries(100, Time.mktime(2011, 1, 1), 86400)

        # Create a XYChart object of size 640 x 400 pixels
        c = ChartDirector::XYChart.new(640, 400)

        # Add a title to the chart using 18 pts Times New Roman Bold Italic font
        c.addTitle("    Product Line Global Revenue", "timesbi.ttf", 18)

        # Set the plotarea at (50, 55) with width 70 pixels less than chart width, and
        # height 90 pixels less than chart height. Use a vertical gradient from light blue
        # (f0f6ff) to sky blue (a0c0ff) as background. Set border to transparent and grid
        # lines to white (ffffff).
        c.setPlotArea(50, 55, c.getWidth() - 70, c.getHeight() - 90,
            c.linearGradientColor(0, 55, 0, c.getHeight() - 35, 0xf0f6ff, 0xa0c0ff), -1,
            ChartDirector::Transparent, 0xffffff, 0xffffff)

        # Add a legend box at (50, 25) using horizontal layout. Use 10pts Arial Bold as
        # font. Set the background and border color to Transparent.
        c.addLegend(50, 25, false, "arialbd.ttf", 10).setBackground(
            ChartDirector::Transparent)

        # Set axis label style to 8pts Arial Bold
        c.xAxis().setLabelStyle("arialbd.ttf", 8)
        c.yAxis().setLabelStyle("arialbd.ttf", 8)

        # Set the axis stem to transparent
        c.xAxis().setColors(ChartDirector::Transparent)
        c.yAxis().setColors(ChartDirector::Transparent)

        # Configure x-axis label format
        c.xAxis().setMultiFormat(ChartDirector::StartOfYearFilter(), "{value|mm/yyyy} ",
            ChartDirector::StartOfMonthFilter(), "{value|mm}")

        # Add axis title using 10pts Arial Bold Italic font
        c.yAxis().setTitle("USD millions", "arialbi.ttf", 10)

        # Add a line layer to the chart using a line width of 2 pixels.
        layer = c.addLineLayer2()
        layer.setLineWidth(2)

        # Add 3 data series to the line layer
        layer.setXData(timeStamps)
        layer.addDataSet(data0, 0xff3333, "Alpha")
        layer.addDataSet(data1, 0x008800, "Beta")
        layer.addDataSet(data2, 0x3333cc, "Gamma")

        # Create the WebChartViewer object
        @viewer = ChartDirector::WebChartViewer.new(request, "chart1")

        # Create the image and save it in a session variable
        session[@viewer.getId()] = c.makeChart2(ChartDirector::PNG)

        # Set the chart URL to the viewer
        @viewer.setImageUrl(url_for(:action => "get_session_data", :id => @viewer.getId(),
            :nocache => rand))

        # Output Javascript chart model to the browser to support tracking cursor
        @viewer.setChartModel(c.getJsChartModel())
    end

end
